;;; make-skram.el

(unless noninteractive
  (error "See GNUmakefile"))

(require 'cl)

(setq vc-handled-backends nil)          ; documented disabling method

(defun make-skram ()
  (emacs-lisp-mode)
  (let ((out (expand-file-name "skram.data"))
        indexed acc form type name bufs)
    (dolist (idx '(Function Variable))
      (with-temp-buffer

        ;; The next two forms are equivalent to:
        ;;   (info (format "(edb.info)%s Index" idx) (current-buffer))
        ;; but two-arg `info' is not yet (2005-01-18) widely available.
        (info (format "(%s)%s Index" (expand-file-name "edb.info" "..") idx))
        (setq buffer-read-only nil)

        (goto-char (point-min))
        (search-forward "* Menu:")
        (while (re-search-forward "^[*] \\([^: ]+\\)[: ]" (point-max) t)
          (pushnew (intern (match-string 1)) indexed))))
    (dolist (file (mapcar 'expand-file-name command-line-args-left))
      (push (find-file file) bufs)
      (goto-char (point-min))
      (while (ignore-errors (setq form (read (current-buffer))))
        (setq type (when (consp form) (symbol-name (car form)))
              name (when type (cadr form)))
        (when (and type name
                   (not (string-match "^define-" type))
                   (string-match "^def" type))
          (setq name (cond ((atom name)
                            name)
                           ((eq 'quote (car name))
                            (cadr name))
                           (t
                            (car name))))
          (pushnew (list name
                         (car form)
                         (when (memq name indexed) t)
                         (list (file-name-nondirectory file)))
                   acc :key 'car))))
    (dolist (buf bufs)
      (with-current-buffer buf
        (let ((file (file-name-nondirectory buffer-file-name))
              case-fold-search)
          (goto-char (point-min))
          (dolist (ent acc)
            (unless (member file (cadddr ent))
              (when (and (search-forward (symbol-name (car ent)) (point-max) t)
                         (memq (char-before (match-beginning 0))
                               '(?( ?) ?' 32)))
                (push file (cadddr ent))
                (goto-char (point-min))))))
        (kill-buffer nil)))
    (erase-buffer)
    (fundamental-mode)
    (dolist (ent acc)
      (insert (format "%s\n:%s\n:%s\n:future unclear\n:"
                      (car ent) (cadr ent)
                      (if (caddr ent) "ok" "maybe")))
      (dolist (file (cadddr ent))
        (insert file "\n"))
      (insert "\n"))
    (let (make-backup-files)
      (write-file out))
    (kill-buffer nil)))

;;; make-skram.el ends here
